<?php
/**
 * @package Dotclear
 * @subpackage Backend
 *
 * Script permettant de réinitialiser les mots de passe Dotclear
 * Version Dotclear compatible : 2.24
 *
 * @copyright Oleksandr Syenchuk & Association Dotclear
 * @copyright GPL-2.0-only
 */

// Ceci est l'adresse IP autorisée à réinitialiser le mot de passe
define('ALLOWIP', '::1');

// Ce fichier est à placer dans le dossier /admin
require dirname(__FILE__) . '/../inc/admin/prepend.php';

class adminPassword
{
    /**
     * Initializes the page.
     */
    public static function init()
    {
        // Check if user is allowed to reset passwords
        if ($_SERVER['REMOTE_ADDR'] !== ALLOWIP) {
            http::head(412, 'Precondition Failed');
            header('Content-Type: text/plain; charset=UTF-8');

            $allow_ip   = ALLOWIP;
            $current_ip = $_SERVER['REMOTE_ADDR'];

            echo <<<HTML_HEAD
                Vous ne pouvez pas accéder à cette page.

                Adresse IP autorisée : {$allow_ip}
                Votre adresse IP : {$current_ip}

                Si vous êtes l'administrateur, modifiez ce fichier pour y définir votre propre adresse IP.
                HTML_HEAD;

            exit;
        }

        header('Content-Type: text/html; charset=UTF-8');

        // Loading locales for detected language
        $dlang = http::getAcceptLanguage();
        if ($dlang !== 'en' && preg_match('/^[a-z]{2}(-[a-z]{2})?$/', $dlang)) {
            l10n::lang($dlang);
            l10n::set(dirname(__FILE__) . '/../locales/' . $dlang . '/main');
        }

        // Users combo
        $users_combo = [];
        $rs          = dcCore::app()->con->select('SELECT user_id FROM ' . DC_DBPREFIX . 'user');
        while ($rs->fetch()) {
            $id               = html::escapeHTML($rs->user_id);
            $users_combo[$id] = $id;
        }
        dcCore::app()->admin->users_combo = $users_combo;
    }

    /**
     * Processes the request(s).
     */
    public static function process()
    {
        dcCore::app()->admin->user_id       = '';
        dcCore::app()->admin->make_super    = false;
        dcCore::app()->admin->show_password = false;

        if (isset($_POST['act_reset'])) {
            dcCore::app()->admin->user_id    = (string) $_POST['user_id'];
            dcCore::app()->admin->make_super = !empty($_POST['make_super']);
        }

        if (dcCore::app()->admin->user_id !== '' && in_array(dcCore::app()->admin->user_id, dcCore::app()->admin->users_combo)) {
            dcCore::app()->admin->user_pwd = crypt::createPassword();

            $cur = dcCore::app()->con->openCursor(DC_DBPREFIX . 'user');

            $cur->user_pwd         = dcCore::app()->auth->crypt(dcCore::app()->admin->user_pwd);
            $cur->user_status      = 1;
            $cur->user_recover_key = null;

            if (dcCore::app()->admin->make_super) {
                $cur->user_super = 1;
            }

            $sql = new dcUpdateStatement();
            $sql->where('user_id = ' . $sql->quote(dcCore::app()->admin->user_id));
            $sql->update($cur);

            dcCore::app()->admin->show_password = true;
        }
    }

    /**
     * Renders the page.
     */
    public static function render()
    {
        $head = <<<HTML_HEAD
            <!DOCTYPE html>
            <html lang="fr">
            <head>
                <meta http-equiv="Content-type" content="text/html; charset=UTF-8;" />
                <meta http-equiv="Content-Style-Type" content="text/css" />
                <meta name="robots" content="noarchive,noindex,nofollow" />
                <link rel="stylesheet" href="style/default.css" type="text/css" media="screen" />
                <title><?php echo %s; ?></title>
            </head>
            HTML_HEAD;

        $body = <<<HTML_BODY
            <body id="dotclear-admin" class="auth">
                <div id="login-screen">
                    <h1><?php echo %s; ?></h1>
                    %s
                </div>
            </body>
            </html>
            HTML_BODY;

        $form = '';
        if (dcCore::app()->admin->show_password) {
            $form = (new formFieldset())
                    ->fields([
                        (new formLegend(__('Your new password'))),
                        (new formPara())
                            ->items([
                                (new formText('strong'))->text(__('Username:')),
                                (new formText())->text(html::escapeHTML(dcCore::app()->admin->user_id)),
                            ]),
                        (new formPara())
                            ->items([
                                (new formText('strong'))->text(__('Password:')),
                                (new formText())->text(html::escapeHTML(dcCore::app()->admin->user_pwd)),
                            ]),
                    ])
                    ->render() .
                (new formPara())
                    ->items([
                        (new formLink())
                            ->href(dcCore::app()->adminurl->get('admin.auth'))
                            ->text(__('Back to login screen')),
                    ])
                    ->render();
        } else {
            $form = (new formForm('reset'))
                ->action(http::getSelfURI())
                ->method('post')
                ->fields([
                    (new formFieldset())
                        ->fields([
                            (new formLegend(__('Password reset'))),
                            (new formPara())
                                ->class('field')
                                ->items([
                                    (new formSelect('user_id'))
                                        ->label(new formLabel(__('User:')))
                                        ->items(dcCore::app()->admin->users_combo)
                                        ->default(dcCore::app()->admin->user_id),
                                ]),
                            (new formPara())
                                ->items([
                                    (new formCheckbox('make_super'))
                                        ->value(1)
                                        ->label((new formLabel(__('Super administrator'), formLabel::INSIDE_TEXT_AFTER))
                                            ->class('classic')),
                                ]),
                        ]),
                    (new formPara())
                        ->items([
                            (new formSubmit('act_reset', __('Change your password'))),
                        ]),
                    (new formHidden('xd_check', dcCore::app()->getNonce())),
                ])
                ->render();
        }

        echo sprintf($head, __('Password reset')) . sprintf($body, html::escapeHTML(DC_VENDOR_NAME), $form);
    }
}

adminPassword::init();
adminPassword::process();
adminPassword::render();
